/************************************************************************************
RMP RT-Component
Copyright (c) 2010, Segway Japan, Ltd.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/
#ifndef JOYSTICKCONTROLLER_H
#define JOYSTICKCONTROLLER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "intellirobotStub.h"

// </rtc-template>

class rmp_status
{
 public:

  //status
  int int_left_wheel;
  int int_right_wheel;

  float battery_voltage;
  float ui_battery_voltage;

  //for internal use
  int int_left_prev;
  int int_right_prev;


  //odometory

  double wheelwidth;
  double wheelradiusL;
  double wheelradiusR;
  double count1rot;
  
  bool odofirst;

  double odox;
  double odoy;
  double odotheta;

  double addx;
  double addy;
  double addtheta;


  coil::TimeValue odotime;
  coil::TimeValue odotime_prev;

  rmp_status()
  {
    memset(this, 0, sizeof(rmp_status));
    odofirst = true;
  }
};



using namespace RTC;

class RMP200Controller
  : public RTC::DataFlowComponentBase
{
 public:
  RMP200Controller(RTC::Manager* manager);
  ~RMP200Controller();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
  virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  // virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:


  // Configuration variable declaration
  // <rtc-template block="config_declare">
  double m_P;
  double m_I;
  double m_D;  
  double m_KPID;  

  double m_Kspeed;
  double m_Kturn;
  double m_Kodoxy;
  double m_Kodotheta;
  double m_Kslip;
  double m_mps2rmp;
  int m_ODOoutput;
  int m_idIIS;
  double m_wheelwidth;
  double m_wheelradiusL;
  double m_wheelradiusR;
  double m_wheelcount1rot;
  double m_MaxV;
  double m_MaxW;
  // </rtc-template>

  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedDoubleSeq m_angle;
  InPort<TimedDoubleSeq> m_angleIn;
  TimedDoubleSeq m_velocity;
  InPort<TimedDoubleSeq> m_velocityIn;
  TimedFloatSeq m_command;
  InPort<TimedFloatSeq> m_commandIn;
  
  TimedULong m_Input;
  InPort<TimedULong> m_InputIn;
  TimedVelocity m_Velocity;
  InPort<TimedVelocity> m_VelocityIn;
  IIS::TimedVelocity m_VelocityIIS;
  InPort<IIS::TimedVelocity> m_VelocityIISIn;
  IIS::TimedVelocity2D m_Velocity2DIIS;
  InPort<IIS::TimedVelocity2D> m_Velocity2DIISIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_torque;
  OutPort<TimedDoubleSeq> m_torqueOut;

  TimedLong m_EncoderL;
  OutPort<TimedLong> m_EncoderLOut;
  TimedLong m_EncoderR;
  OutPort<TimedLong> m_EncoderROut;
  TimedOdometry m_Odometry;
  OutPort<TimedOdometry> m_OdometryOut;
  RTC::TimedOdometry m_OdometryB;
  OutPort<RTC::TimedOdometry> m_OdometryBOut;

  IIS::TimedVelocity m_RobotVelIIS;
  OutPort<IIS::TimedVelocity> m_RobotVelIISOut;
  IIS::TimedPosition m_RobotPosIIS;
  OutPort<IIS::TimedPosition> m_RobotPosIISOut;
  IIS::TimedVelocity2D m_RobotVel2DIIS;
  OutPort<IIS::TimedVelocity2D> m_RobotVel2DIISOut;
  IIS::TimedPose2D m_RobotPose2DIIS;
  OutPort<IIS::TimedPose2D> m_RobotPose2DIISOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;
  // <rtc-template block="private_attribute">
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

  double tireVelFL;
  double tireVelFR;
  double tireVelRL;
  double tireVelRR;
  double tireVelFLint;
  double tireVelFRint;
  double tireVelRLint;
  double tireVelRRint;

  //Torque output for virtual robot
  void rmp_set_speed(int l, int r);
  //Encoder input from virtual robot
  void rmp_get_encoder2(double count1rot, int &le, int &re);


  //Blackship.cppƓ
  unsigned long Stop;

  int Speed;
  int Turn;
  rmp_status Status;

  coil::TimeValue curtime;
};


extern "C"
{
  void RMP200ControllerInit(RTC::Manager* manager);
};

#endif // JOYSTICKCONTROLLER_H
