/************************************************************************************
RMP RT-Component
Copyright (c) 2010, Segway Japan, Ltd.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/
#ifndef __RMP_IF_H__
#define __RMP_IF_H__



class rmp_status
{
 public:

  //status 0x403
  int int_left_wheel;
  int int_right_wheel;
  coil::TimeValue tv403;

  //status 0x401
  float pitchangle;
  float pitchrate;
  float rollangle;
  float rollrate;
  coil::TimeValue tv401;

  //status 0x402
  float yawrate;
  float lwvel;
  float rwvel;
  coil::TimeValue tv402;

  //status 0x406
  float battery_voltage;
  float ui_battery_voltage;
  coil::TimeValue tv406;

  //for internal use
  int int_left_prev;
  int int_right_prev;


  //odometory

  double wheelwidth;
  double wheelradiusL;
  double wheelradiusR;
  double count1rot;
  
  bool odofirst;
  bool odogot;

  double odox;
  double odoy;
  double odotheta;

  double addx;
  double addy;
  double addtheta;
  coil::TimeValue odotime;
  coil::TimeValue odotime_prev;

  rmp_status()
  {
    memset(this, 0, sizeof(rmp_status));
    odofirst = true;
    odogot = false;
  }
};

/*!
  @brief open RMP interface
  @param[in] device port for RMP. 
            'FTDI'          : use real RMP
	    "/dev/ttySIM*"  : use simulated RMP

  @param[in] ftdino
            FTDI device no of rmp  
*/
void rmp_open(char* port, rmp_status& status);


/*!
  @brief close RMP interface
*/
void rmp_close();

/*!
  @brief get status from RMP
  @param[in] pointer of status
*/
void rmp_getstatus(rmp_status& status, double kodoxy, double kodotheta);

/*!
  @brief send set speed to RMP
  @param[in] speed     frontspeed 
  @param[in] turn      turn rate
*/
void rmp_setspeed(short speed, short turn, unsigned short command = 0, unsigned short param = 0);


enum {
  COMMAND_SET_MAX_VELOCITY_SCALE_FACTOR         = 10,//0-16 (0-1.0)
  COMMAND_SET_MAXIMUM_ACCELERATION_SCALE_FACTOR = 11,//0-16 (0-1.0)
  COMMAND_SET_MAXIMUM_TURN_RATE_SCALE_FACTOR    = 12,//0-16 (0-1.0)
  COMMAND_GAIN_SCHEDULE                         = 13,//0,1,2
  COMMAND_CURRENT_LIMIT_SCALE_FACTOR            = 14,//0-256 (0-1.0)
  COMMAND_BALANCE_MODE_LOCKOUT                  = 15,//0:BalanceMode 1:LockoutBalance
  COMMAND_SET_OPERATIONAL_MODE                  = 16,//1:Run tractor mode 2:Run balance mode 3:Power Down
  COMMAND_RESET_INTEGRATORS                     = 50,
};
void rmp_setconfiguration(unsigned short command, unsigned short param);







#endif
